<?php

namespace Mediavine\Create\Importer;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class Admin_Init {
	function admin_enqueue_scripts( $hook ) {
		$settings = \Mediavine\Settings::get_settings();
		$shapes   = \Mediavine\Create\Shapes::get_shapes();

		wp_enqueue_style( Plugin::PLUGIN_DOMAIN . '/mv-recipe-importer.css', Plugin::assets_url() . 'admin/ui/build/app.build.' . Plugin::VERSION . '.css' );

		$script_url = Plugin::assets_url() . 'admin/ui/build/app.build.' . Plugin::VERSION . '.js';

		if ( defined( 'ENV' ) ) {
			$script_url = '//localhost:3002/app.build.' . Plugin::VERSION . '.js';
			wp_dequeue_style( Plugin::PLUGIN_DOMAIN . '/mv-recipe-importer.css' );
		}

		wp_register_script( Plugin::PLUGIN_DOMAIN . '/mv-recipe-importer.js', $script_url, [ 'mv_raven', 'mv_create/mv-create.js' ], Plugin::VERSION, true );

		wp_localize_script(
			Plugin::PLUGIN_DOMAIN . '/mv-recipe-importer.js', 'MV_IMPORTER', [
				'__URL__'       => esc_url_raw( rest_url() ),
				'__NONCE__'     => wp_create_nonce( 'wp_rest' ),
				'__ADMIN_URL__' => esc_url_raw( admin_url() ),
				'imported'      => json_decode( MV_Recipe_Importer::get_imported_recipes() ),
				'replaced'      => json_decode( MV_Recipe_Importer::get_replaced_recipes() ),
			]
		);
		wp_enqueue_script( Plugin::PLUGIN_DOMAIN . '/mv-recipe-importer.js' );
	}

	function media_buttons( $editor_id ) {
		if ( 'content' !== $editor_id ) {
			return;
		}
		?>
			<div id="MVSingleImporter" class="mv-ui-root"></div>
		<?php
	}

	function submenu_page() {
		return '';
	}

	function update_admin_menu() {
		add_submenu_page(
			'edit.php?post_type=mv_create',
			__( 'Import Recipes', 'mediavine' ),
			__( 'Import Recipes', 'mediavine' ),
			'manage_options',
			'import',
			[ $this, 'submenu_page' ]
		);
	}

	function enqueue_block_editor_assets() {
		$handle     = 'mv-i-blocks';
		$script_url = plugins_url( 'admin/ui/build/blocks.build.' . Plugin::VERSION . '.js', __DIR__ );

		if ( defined( 'ENV' ) ) {
			$script_url = '//localhost:3031/blocks.build.' . Plugin::VERSION . '.js';
		}

		wp_enqueue_script(
			$handle,
			$script_url,
			[ 'wp-blocks', 'wp-i18n', 'wp-element', 'mv_create/mv-create.js' ], // Gutenberg dependencies
			Plugin::VERSION,
			true
		);
	}

	function import_notice() {
		?>
			<div id="mv-gb-import-instructions" style="display: none;">
				<div class="mv-gb-instructions">
					<h2>Importing recipes?</h2>

					<p>
						<a
							href="https://help.mediavine.com/create-by-mediavine/recipe-imports/importing-recipes-through-gutenberg"
							target="_blank">
							Here are instructions how.
						</a>
					</p>

					<button class="button" id="mv-gb-dismiss">Got it!</button>
				</div>
			</div>
		<?php
	}

	function init() {
		add_action( 'admin_menu', [ $this, 'update_admin_menu' ] );
		add_action( 'media_buttons', [ $this, 'media_buttons' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );
		add_action( 'enqueue_block_editor_assets', [ $this, 'enqueue_block_editor_assets' ] );
		add_action( 'admin_footer', [ $this, 'import_notice' ] );
		add_action( 'admin_notices', [ $this, 'upgrade_notice' ] );
	}

	/**
	 * Displays an upgrade notice for the importer plugin if it's out of date
	 * plugin because we need the notice for people with out of date importers,
	 * and that would mean an importer update wouldn't happen.
	 *
	 * @return void
	 */
	public function upgrade_notice() {
		if (
			class_exists( 'Mediavine\Create\Plugin' ) &&
			version_compare( Plugin::VERSION, \Mediavine\Create\Plugin::REQUIRED_IMPORTER_VERSION, '<' )
		) {
			$install_url   = wp_nonce_url(
				add_query_arg(
					[
						'action' => 'upgrade-plugin',
						'plugin' => 'mediavine-recipe-importers',
					],
					admin_url( 'update.php' )
				),
				'upgrade-plugin_' . 'mediavine-recipe-importers'
			);
			$activate_text = sprintf(
				'%1$s<br>%2$s',
				sprintf(
					// translators: the placeholder signifies the version of Create required by the Importers plugin
					__( '<strong>Mediavine Recipe Importer</strong> is out of date. Please upgrade to at least version <strong>%s</strong>.', 'mediavine' ),
					\Mediavine\Create\Plugin::REQUIRED_IMPORTER_VERSION
				),
				sprintf(
					/* translators: plugin installation url */
					'<a href="%s">Click here to update Mediavine Recipe Importer</a>',
					$install_url
				)
			);
			echo wp_kses_post( '<div class="error"><p>' . $activate_text . '</p></div>' );
		}
	}
}
