<?php

namespace Mediavine\Create\Importer;

require_once plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( class_exists( 'Mediavine\Create\Plugin' ) ) {

	class Plugin extends \Mediavine\Create\Plugin {

		const VERSION = '0.10.8';

		const DB_VERSION = '1.0.0';

		const TEXT_DOMAIN = 'mediavine';

		const PLUGIN_DOMAIN = 'mv_recipe_cards_importer';

		const PREFIX = '_mv_';

		const PLUGIN_FILE_PATH = __FILE__;

		private static $instance = null;

		public static $db_interface = null;

		public static $api_services = null;

		public static $models = null;

		public $api_route = 'mv-recipes';

		public $api_version = 'v1';

		public $rest_response = null;

		public $recipe_importer;
		public $admin_init;

		public static function get_instance() {
			if ( null === self::$instance ) {
				self::$instance = new self;
				self::$instance->init();
			}
			return self::$instance;
		}

		function __construct() {
			$Create                = \Mediavine\Create\Plugin::get_instance();
			self::$models          = \Mediavine\MV_DBI::get_models(
				[
					'mv_creations',
					'mv_supplies',
					'mv_nutrition',
					'mv_reviews',
					'mv_settings',
				]
			);
			self::$api_services    = \Mediavine\API_Services::get_instance();
			$this->recipe_importer = new MV_Recipe_Importer();
			$this->admin_init      = new Admin_Init();

		}

		public static function assets_url() {
			return plugin_dir_url( __FILE__ );
		}

		public function init() {
			$this->recipe_importer->fix_mpp_reviews();
			$this->fix_wpurp_cuisine_imports();
			$this->fix_tasty_cuisine_keywords_imports();
		}

		/**
		 * Fixes cuisines that were imported as integers instead of text from WPURP.
		 *
		 * Remove March 2020
		 *
		 * @since 0.10.6
		 *
		 * @return void
		 */
		public function fix_wpurp_cuisine_imports() {
			global $wpdb;
			$last_plugin_version = get_option( 'mv_create_version', Plugin::VERSION );
			if ( version_compare( $last_plugin_version, '1.4.19', '<' ) ) {
				$creations = $wpdb->get_results(
					"SELECT id, category, secondary_term, metadata
						FROM {$wpdb->prefix}mv_creations
						WHERE type='recipe'
							AND metadata LIKE '%wp_ultimate%'
							AND metadata NOT LIKE '%fixed_wpurp_cuisine%'",
					ARRAY_A
				);
				$ids       = [];
				foreach ( $creations as $creation ) {
					if ( ! empty( $creation['secondary_term'] ) && is_numeric( $creation['secondary_term'] ) ) {
						$term = get_term( $creation['secondary_term'], 'cuisine' );
						if ( $term && ! is_wp_error( $term ) ) {
							$creation['secondary_term'] = $term->name;
						}
					}
					if ( ! empty( $creation['category'] ) && is_numeric( $creation['category'] ) ) {
						$term = get_term( $creation['category'], 'category' );
						if ( $term && ! is_wp_error( $term ) ) {
							$creation['category'] = $term->name;
						}
					}
					$metadata                      = json_decode( $creation['metadata'] );
					$metadata->fixed_wpurp_cuisine = true;
					$creation['metadata']          = wp_json_encode( $metadata );
					self::$models_v2->mv_creations->update( $creation );
					$ids[] = $creation['id'];
				}
				\Mediavine\Create\Publish::update_publish_queue( $ids );
			}
		}

		/**
		 * Fixes missing cuisine and keyword imports from Tasty recipes.
		 *
		 * Previous imports from Tasty might be missing keywords and cuisines.
		 *
		 * Remove March 2020
		 *
		 * @since 0.10.6
		 *
		 * @return void
		 */
		public function fix_tasty_cuisine_keywords_imports() {
			global $wpdb;
			$last_plugin_version = get_option( 'mv_create_version', Plugin::VERSION );

			if ( version_compare( $last_plugin_version, '1.4.18', '<' ) ) {
				$creations = $wpdb->get_results(
					"SELECT id, original_post_id, secondary_term, keywords, metadata
						FROM {$wpdb->prefix}mv_creations
						WHERE type='recipe'
							AND original_post_id IS NOT NULL
							AND (
								secondary_term IS NULL
								OR keywords IS NULL
							)
							AND metadata LIKE '%tasty%'
							AND metadata NOT LIKE '%fixed_tasty_cuisine_keywords%'",
					ARRAY_A
				);
				$ids       = [];
				foreach ( $creations as $creation ) {
					$post = get_post( $creation['original_post_id'] );
					if ( empty( $creation['secondary_term'] ) ) {
						$cuisine = get_post_meta( $post->ID, 'cuisine', true );
						if ( $cuisine ) {
							$cuisine_id                 = $this->recipe_importer->upsert_cuisine( $cuisine );
							$creation['secondary_term'] = $cuisine_id;
						}
					}
					if ( empty( $creation['keywords'] ) ) {
						$keywords = get_post_meta( $post->ID, 'keywords', true );
						if ( $keywords ) {
							$creation['keywords'] = $keywords;
						}
					}
					$metadata                               = json_decode( $creation['metadata'] );
					$metadata->fixed_tasty_cuisine_keywords = true;
					$creation['metadata']                   = wp_json_encode( $metadata );
					self::$models_v2->mv_creations->update( $creation );
					$ids[] = $creation['id'];
				}
				\Mediavine\Create\Publish::update_publish_queue( $ids );
			}
		}

	}

	$Plugin = Plugin::get_instance();
	$Plugin->recipe_importer->init();
	$Plugin->admin_init->init();
}
