<?php
/**
 * The plugin bootstrap file
 *
 * @link              https://www.mediavine.com/
 * @since             1.0.0
 * @package           Mediavine/recipe_card
 *
 * Plugin Name:       Mediavine Recipe Importer
 * Plugin URI:        https://www.mediavine.com/
 * Description:       Migrates recipes from other plugins to Create by Mediavine.
 * Version:           0.10.8
 * Author:            Mediavine
 * Author URI:        https://www.mediavine.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       mediavine
 * Domain Path:       /languages
 */

// Prevent direct access
if ( ! defined( 'ABSPATH' ) ) {
	exit( 'This plugin requires WordPress' );
}

require 'plugin_update_check.php';
$MyUpdateChecker = new PluginUpdateChecker_2_0(
	'https://kernl.us/api/v1/updates/5b104641a89fe95d1d1748c1/',
	__FILE__,
	'mediavine-create-recipe-importers',
	1
);

function mv_recipe_importer_is_compatible() {
	global $wp_version;
	$wp         = '4.7';
	$php        = '5.6.20';
	$compatible = true;

	if ( version_compare( PHP_VERSION, $php ) < 0 ) {
		$compatible = false;
	}

	if ( version_compare( $wp_version, $wp ) < 0 ) {
		$compatible = false;
	}

	return $compatible;
}

function mv_get_plugins() {
	if ( ! function_exists( 'get_plugins' ) ) {
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
	}

	return get_plugins();
}

function is_plugin_installed( $plugin ) {
	if ( array_key_exists( $plugin, mv_get_plugins( 'mediavine-create/mediavine-create.php' ) ) ) {
		return true;
	}

	return false;
}

function is_plugin_up_to_date() {
	$required_create_version = '1.6.1';
	if (
		class_exists( 'Mediavine\Create\Plugin' ) &&
		version_compare( \Mediavine\Create\Plugin::VERSION, $required_create_version, '>=' )
	) {
		return true;
	}
	return false;
}

/**
 * Check if PHP and WP are updated enough and if Create by Mediavine is
 * installed
 * @return void
 */
function mv_recipe_importer_incompatible_notice() {
	if ( ! mv_recipe_importer_is_compatible() ) {
		printf(
			'<div class="notice notice-error"><p>%1$s</p></div>',
			wp_kses_post( __( '<strong>Mediavine Recipe Importer</strong> requires PHP 5.6.20 or higher WordPress 4.7 or higher. Please upgrade your hosting and/or WordPress.', 'mediavine' ) )
		);
		printf(
			'<div class="notice notice-error"><p><em>%1$s</em></p></div>',
			wp_kses_post( __( 'The plugin has been deactivated.', 'mediavine' ) )
		);
		deactivate_plugins( plugin_basename( __FILE__ ) );
		return;
	}

	if ( ! class_exists( 'Mediavine\Create\Plugin' ) ) {
		$install_url   = wp_nonce_url(
			add_query_arg(
				[
					'action' => 'install-plugin',
					'plugin' => 'mediavine-create',
				],
				admin_url( 'update.php' )
			),
			'install-plugin_' . 'mediavine-create'
		);
		$activate_text = sprintf(
			'%1$s<br>%2$s',
			__( '<strong>Create by Mediavine</strong> is required to run <strong>Mediavine Recipe Importer</strong>', 'mediavine' ),
			/* translators: plugin installation url */
			sprintf(
				'<a href="%s">Click here to install Create by Mediavine</a>',
				$install_url
			)
		);
		$plugin_slug = 'mediavine-create/mediavine-create.php';

		if ( is_plugin_installed( $plugin_slug ) ) {
			$activate_url  = wp_nonce_url(
				add_query_arg(
					[
						'action' => 'activate',
						'plugin' => $plugin_slug,
					],
					admin_url( 'plugins.php' )
				),
				'activate-plugin_' . $plugin_slug
			);
			$activate_text = sprintf(
				'%1$s<br>%2$s',
				__( '<strong>Create by Mediavine</strong> is required to run <strong>Mediavine Recipe Importer</strong>. The functionality of this plugin has been disabled to prevent your site from breaking.', 'mediavine' ),
				/* translators: plugin installation url */
				sprintf(
					'<a href="%s">Click here to activate Create by Mediavine</a>',
					$activate_url
				)
			);
		}

		echo wp_kses_post( '<div class="error"><p>' . $activate_text . '</p></div>' );
	} elseif ( ! is_plugin_up_to_date() ) {
		$required_create_version = '1.6.1';
		$activate_text           = sprintf(
			// translators: the placeholder signifies the version of Create required by the Importers plugin
			__( '<strong>Create by Mediavine</strong> must be updated to %s to work with <strong>Mediavine Recipe Importer</strong>.', 'mediavine' ),
			$required_create_version
		);
		echo wp_kses_post( '<div class="error"><p>' . $activate_text . '</p></div>' );
	}
}

/**
 * Activate Create by Mediavine on activation if it exists
 * @return void
 */
function mv_recipe_importer_activate_recipe_card() {
	$plugin_slug = 'mediavine-create/mediavine-create.php';

	if ( is_plugin_installed( $plugin_slug ) && is_plugin_up_to_date() ) {
		activate_plugin( $plugin_slug );
	}
}

add_action( 'admin_notices', 'mv_recipe_importer_incompatible_notice' );
register_activation_hook( __FILE__, 'mv_recipe_importer_activate_recipe_card' );

// Only run plugin if Create by Mediavine is activated
if ( class_exists( 'Mediavine\Create\Plugin' ) && is_plugin_up_to_date() ) {
	require_once( 'class-plugin.php' );
}

add_action( 'wp_head', 'mv_preclude_other_json_ld' );
function mv_preclude_other_json_ld() {
	global $post;
	if ( $post && strpos( $post->post_content, 'mv_recipe' ) === false ) {
		return false;
	}
	if ( class_exists( '\SimpleRecipePro\Recipes_Schema' ) ) {
		$_GET['print'] = true;
	}
}
